==> hooks-0.2.0.tgz: package/lib/useSession.js <==
import { useCallback, useContext, useEffect, useMemo, useState } from "react";
import { CanvasContext } from "./CanvasContext.js";
import { getSessionObject, setSessionObject, removeSessionObject } from "./sessionKeyStorage.js";
const second = 1000;
const minute = 60 * second;
const hour = 60 * minute;
const BLOCK_CACHE_TIME = 1.5 * second;
let cachedBlock = null;
const getLatestBlockWithCache = async (chainImplementation) => {
    if (cachedBlock !== null && cachedBlock[1] > +new Date() - BLOCK_CACHE_TIME) {
        return cachedBlock[0];
    }
    else {
        const block = await chainImplementation.getLatestBlock();
        cachedBlock = [block, +new Date()];
        return block;
    }
};
export function useSession(chainImplementation, signer, options = {}) {
    const { chain, chainId } = chainImplementation;
    const { host, data } = useContext(CanvasContext);
    const [isLoading, setIsLoading] = useState(true);
    const [isPending, setIsPending] = useState(false);
    const [sessionSigner, setSessionSigner] = useState(null);
    const [sessionAddress, setSessionAddress] = useState(null);
    const [sessionExpiration, setSessionExpiration] = useState(null);
    const loadSavedSession = useCallback(async (data, signer) => {
        const signerAddress = await chainImplementation.getSignerAddress(signer);
        const sessionObject = getSessionObject(chain, chainId, signerAddress);
        if (sessionObject !== null) {
            if (sessionObject.app !== data.uri || sessionObject.expiration < Date.now()) {
                removeSessionObject(chain, chainId, signerAddress);
            }
            else {
                const delegatedSigner = chainImplementation.importDelegatedSigner(sessionObject.sessionPrivateKey);
                const sessionAddress = await chainImplementation.getDelegatedSignerAddress(delegatedSigner);
                const res = await fetch(`${host}/sessions`, {
                    method: "POST",
                    headers: { "Content-Type": "application/json" },
                    body: JSON.stringify({
                        hasSession: sessionAddress,
                        chain: chain,
                        chainId: chainId,
                    }),
                });
                const { hasSession } = await res.json();
                if (hasSession) {
                    setSessionAddress(sessionAddress);
                    setSessionSigner(delegatedSigner);
                    setSessionExpiration(sessionObject.expiration);
                    setIsLoading(false);
                    return;
                }
            }
        }
        setSessionAddress(null);
        setSessionSigner(null);
        setSessionExpiration(null);
        setIsLoading(false);
    }, []);
    useEffect(() => {
        if (host === null || data === null || signer === null || signer === undefined) {
            return;
        }
        loadSavedSession(data, signer);
    }, [host, data, signer]);
    const login = useCallback(async () => {
        if (host === null) {
            throw new Error("no host configured");
        }
        else if (data === null) {
            throw new Error("login() called before a connection to the Canvas node was established");
        }
        else if (signer === null || signer === undefined) {
            throw new Error("login() called without a signer");
        }
        else if (isPending) {
            throw new Error("another login() call is already pending");
        }
        try {
            setIsPending(true);
            const signerAddress = await chainImplementation.getSignerAddress(signer);
            const sessionIssued = Date.now();
            const sessionDuration = options.sessionDuration ?? 24 * hour;
            const delegatedSigner = await chainImplementation.generateDelegatedSigner();
            const sessionAddress = await chainImplementation.getDelegatedSignerAddress(delegatedSigner);
            const sessionObject = {
                app: data.uri,
                sessionPrivateKey: chainImplementation.exportDelegatedSigner(delegatedSigner),
                expiration: sessionIssued + sessionDuration,
            };
            const block = options.unchecked ? null : await chainImplementation.getLatestBlock();
            const session = await chainImplementation.signSession(signer, {
                from: signerAddress,
                app: data.uri,
                appName: data.appName,
                sessionAddress,
                sessionDuration,
                sessionIssued,
                block,
                chain,
                chainId,
            });
            const res = await fetch(`${host}/sessions`, {
                method: "POST",
                headers: { "Content-Type": "application/json" },
                body: JSON.stringify(session),
            });
            if (!res.ok) {
                const message = await res.text();
                throw new Error(message);
            }
            setSessionObject(chain, chainId, signerAddress, sessionObject);
            setSessionSigner(delegatedSigner);
            setSessionAddress(sessionAddress);
            setSessionExpiration(sessionObject.expiration);
        }
        finally {
            setIsPending(false);
        }
    }, [signer, host, data, isPending]);
    const logout = useCallback(async () => {
        if (signer) {
            const signerAddress = await chainImplementation.getSignerAddress(signer);
            removeSessionObject(chain, chainId, signerAddress);
        }
        setSessionSigner(null);
        setSessionAddress(null);
        setSessionExpiration(null);
    }, [signer]);
    const dispatch = useCallback(async (call, callArgs, callOptions) => {
        if (host === null) {
            throw new Error("no host configured");
        }
        else if (data === null) {
            throw new Error("dispatch() called before the application connection was established");
        }
        else if (signer === null || signer === undefined) {
            throw new Error("dispatch() called without a signer");
        }
        else if (sessionSigner === null || sessionExpiration === null) {
            throw new Error("dispatch() called before login");
        }
        else if (sessionExpiration < Date.now()) {
            throw new Error("session expired, please log in again");
        }
        const block = options.unchecked ? null : await getLatestBlockWithCache(chainImplementation);
        const action = await chainImplementation.signDelegatedAction(sessionSigner, {
            app: data.uri,
            appName: data.appName,
            from: await chainImplementation.getSignerAddress(signer),
            call,
            callArgs,
            chain,
            chainId,
            timestamp: callOptions?.timestamp ?? Date.now(),
            block: options.unchecked ? null : block,
        });
        const res = await fetch(`${host}/actions`, {
            method: "POST",
            headers: { "Content-Type": "application/json" },
            body: JSON.stringify(action),
        });
        if (res.ok) {
            return await res.json();
        }
        else {
            const message = await res.text();
            if (message === "session not found" || message === "session expired") {
                const signerAddress = await chainImplementation.getSignerAddress(signer);
                removeSessionObject(chain, chainId, signerAddress);
                setSessionSigner(null);
                setSessionAddress(null);
                setSessionExpiration(null);
            }
            throw new Error(message);
        }
    }, [host, data, signer, sessionSigner, sessionExpiration]);
    const client = useMemo(() => data &&
        Object.fromEntries(data.actions.map((action) => [action, (args, callOptions) => dispatch(action, args, callOptions)])), [data, dispatch]);
    return {
        isLoading,
        isPending,
        sessionAddress,
        sessionExpiration,
        login,
        logout,
        client: signer && sessionSigner && Date.now() < (sessionExpiration ?? 0) ? client : null,
    };
}

==> hooks-0.3.2.tgz: package/lib/useSession.js <==
import { useCallback, useContext, useEffect, useMemo, useState } from "react";
import { InvalidChainError, InvalidChainIdError, } from "@canvas-js/interfaces";
import { CanvasContext } from "./CanvasContext.js";
import { getSessionObject, setSessionObject, removeSessionObject } from "./sessionKeyStorage.js";
const second = 1000;
const minute = 60 * second;
const hour = 60 * minute;
const BLOCK_CACHE_TIME = 1.5 * second;
let cachedBlock = null;
const getLatestBlockWithCache = async (chainImplementation) => {
    if (cachedBlock !== null && cachedBlock[1] > +new Date() - BLOCK_CACHE_TIME) {
        return cachedBlock[0];
    }
    else {
        const block = await chainImplementation.getLatestBlock();
        cachedBlock = [block, +new Date()];
        return block;
    }
};
export function useSession(chainImplementation, signer, options = {}) {
    const { chain, chainId } = chainImplementation;
    const { api, data } = useContext(CanvasContext);
    const [isLoading, setIsLoading] = useState(true);
    const [isPending, setIsPending] = useState(false);
    const [sessionSigner, setSessionSigner] = useState(null);
    const [sessionAddress, setSessionAddress] = useState(null);
    const [sessionExpiration, setSessionExpiration] = useState(null);
    const loadSavedSession = useCallback(async (data, signer) => {
        const signerAddress = await chainImplementation.getSignerAddress(signer);
        const sessionObject = getSessionObject(chain, chainId, signerAddress);
        if (sessionObject !== null) {
            if (sessionObject.app !== data.uri || sessionObject.expiration < Date.now()) {
                removeSessionObject(chain, chainId, signerAddress);
            }
            else {
                const delegatedSigner = chainImplementation.importDelegatedSigner(sessionObject.sessionPrivateKey);
                const sessionAddress = await chainImplementation.getDelegatedSignerAddress(delegatedSigner);
                setSessionAddress(sessionAddress);
                setSessionSigner(delegatedSigner);
                setSessionExpiration(sessionObject.expiration);
                setIsLoading(false);
                return;
            }
        }
        setSessionAddress(null);
        setSessionSigner(null);
        setSessionExpiration(null);
        setIsLoading(false);
    }, []);
    useEffect(() => {
        if (api === null || data === null || signer === null || signer === undefined) {
            return;
        }
        loadSavedSession(data, signer);
    }, [api, data, signer]);
    const login = useCallback(async () => {
        if (api === null) {
            throw new Error("no Core API provider configured");
        }
        else if (data === null) {
            throw new Error("login() called before a connection to the Canvas node was established");
        }
        else if (signer === null || signer === undefined) {
            throw new Error("login() called without a signer");
        }
        else if (isPending) {
            throw new Error("another login() call is already pending");
        }
        try {
            setIsPending(true);
            const signerAddress = await chainImplementation.getSignerAddress(signer);
            const sessionIssued = Date.now();
            const sessionDuration = options.sessionDuration ?? 24 * hour;
            const delegatedSigner = await chainImplementation.generateDelegatedSigner();
            const sessionAddress = await chainImplementation.getDelegatedSignerAddress(delegatedSigner);
            const sessionObject = {
                app: data.uri,
                sessionPrivateKey: chainImplementation.exportDelegatedSigner(delegatedSigner),
                expiration: sessionIssued + sessionDuration,
            };
            const block = options.unchecked ? null : await chainImplementation.getLatestBlock();
            const supportedChainIds = data.chains[chain];
            if (supportedChainIds === undefined) {
                throw new InvalidChainError(`Invalid chain: ${chain}:${chainId}`);
            }
            if (!supportedChainIds.includes(chainId)) {
                throw new InvalidChainIdError(`Invalid chain ID: ${chain}:${chainId}`);
            }
            const session = await chainImplementation.signSession(signer, {
                from: signerAddress,
                app: data.uri,
                appName: data.appName,
                sessionAddress,
                sessionDuration,
                sessionIssued,
                block,
                chain,
                chainId,
            });
            await api.apply(session);
            setSessionObject(chain, chainId, signerAddress, sessionObject);
            setSessionSigner(delegatedSigner);
            setSessionAddress(sessionAddress);
            setSessionExpiration(sessionObject.expiration);
        }
        finally {
            setIsPending(false);
        }
    }, [signer, api, data, isPending]);
    const logout = useCallback(async () => {
        if (signer) {
            const signerAddress = await chainImplementation.getSignerAddress(signer);
            removeSessionObject(chain, chainId, signerAddress);
        }
        setSessionSigner(null);
        setSessionAddress(null);
        setSessionExpiration(null);
    }, [signer]);
    const dispatch = useCallback(async (call, callArgs, callOptions) => {
        if (api === null) {
            throw new Error("no Core API connection configured");
        }
        else if (data === null) {
            throw new Error("dispatch() called before the application connection was established");
        }
        else if (signer === null || signer === undefined) {
            throw new Error("dispatch() called without a signer");
        }
        else if (sessionSigner === null || sessionExpiration === null) {
            throw new Error("dispatch() called before login");
        }
        else if (sessionExpiration < Date.now()) {
            throw new Error("session expired, please log in again");
        }
        const block = options.unchecked ? null : await getLatestBlockWithCache(chainImplementation);
        const action = await chainImplementation.signDelegatedAction(sessionSigner, {
            app: data.uri,
            appName: data.appName,
            from: await chainImplementation.getSignerAddress(signer),
            call,
            callArgs,
            chain,
            chainId,
            timestamp: callOptions?.timestamp ?? Date.now(),
            block: options.unchecked ? null : block,
        });
        try {
            return api.apply(action);
        }
        catch (err) {
            if (err instanceof Error) {
                if (err.message === "session not found" || err.message === "session expired") {
                    const signerAddress = await chainImplementation.getSignerAddress(signer);
                    removeSessionObject(chain, chainId, signerAddress);
                    setSessionSigner(null);
                    setSessionAddress(null);
                    setSessionExpiration(null);
                }
            }
            throw err;
        }
    }, [api, data, signer, sessionSigner, sessionExpiration]);
    const client = useMemo(() => data &&
        Object.fromEntries(data.actions.map((action) => [action, (args, callOptions) => dispatch(action, args, callOptions)])), [data, dispatch]);
    return {
        isLoading,
        isPending,
        sessionAddress,
        sessionExpiration,
        login,
        logout,
        client: signer && sessionSigner && Date.now() < (sessionExpiration ?? 0) ? client : null,
    };
}
